/*
 * sprite.hpp - XD-Ball
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#ifndef SPRITE_HPP_INCLUDED
#define SPRITE_HPP_INCLUDED

#include "opengl.hpp"

#include <string>

#include <SFML/System/Vector2.hpp>
#include <SFML/Graphics/Color.hpp>

#include "texture.hpp"
#include "renderable.hpp"

class Sprite: public Renderable
{
private:
	Texture* texture_ptr;
	static GLuint display_list;
	static unsigned int num_sprites;
	
	void make_list();
	
public:
	Sprite(const std::string&, const sf::Vector2f&, const sf::Vector2f&,
		const sf::Vector2f& = sf::Vector2f(0, 0), float = 0, const sf::Color& = sf::Color(255, 255, 255, 255));
	Sprite(const std::string&, const sf::Vector2f& = sf::Vector2f(0, 0));
	
	sf::Vector2f position;
	sf::Vector2f size;
	sf::Vector2f origin;
	float angle;
	sf::Color color;
	
	virtual void render();
	
	bool intersects(const Sprite&);
	
	virtual ~Sprite();
};

#endif // SPRITE_HPP_INCLUDED
