/*
 * ball.cpp - XD-Ball
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "ball.hpp"

#include <cmath>
#include <iostream>

#include "opengl.hpp"
#include "resources.hpp"

Ball::Ball(const sf::Vector2f& p, const sf::Vector2f& siz, const sf::Vector2f& spe):
	Sprite("BALL", p, siz, siz / 2.0f), speed(spe)
{
	
}

void Ball::update(float dt)
{
	position += speed * dt;
}

Ball::Collision Ball::collide(const Brick* brick)
{
	sf::Vector2f old_speed = speed;
	Collision ret;
	ret.happened = false;
	
	if ((position.x >= brick->position.x) && (position.x <= (brick->position.x + brick->size.x)))
	{
		if (position.y <= (brick->position.y + (brick->size.y / 2.0f))) // alatta
		{
			if ((position.y + (size.y / 2.0f)) >= brick->position.y)
			{
				speed.y = -fabs(speed.y);
				ret.happened = true;
				ret.position = sf::Vector2f(position.x, position.y + size.y / 2.0f);
			}
		}
		else // fölötte
		{
			if ((position.y - (size.y / 2.0f)) <= (brick->position.y + brick->size.y))
			{
				speed.y = fabs(speed.y);
				ret.happened = true;
				ret.position = sf::Vector2f(position.x, position.y - size.y / 2.0f);
			}
		}
	}
	else
	{
		if ((position.y >= brick->position.y) && (position.y <= (brick->position.y + brick->size.y)))
		{
			if (position.x <= (brick->position.x + (brick->size.x / 2.0f))) // balra
			{
				if ((position.x + (size.x / 2.0f)) >= brick->position.x)
				{
					speed.x = -fabs(speed.x);
					ret.happened = true;
					ret.position = sf::Vector2f(position.x + size.x / 2.0f, position.y);
				}
			}
			else // jobbra
			{
				if ((position.x - (size.x / 2.0f)) <= (brick->position.x + brick->size.x))
				{
					speed.x = fabs(speed.x);
					ret.happened = true;
					ret.position = sf::Vector2f(position.x - size.x / 2.0f, position.y);
				}
			}
		}
		else
		{
			// 4 sarok
			
			float sqr_radius = size.x * size.x * 0.25f; // ez tuti jó
			
			
			sf::Vector2f corners[4];
			
			corners[0] = corners[1] = corners[2] = corners[3] = brick->position;
			corners[1].x += brick->size.x;
			corners[2].y += brick->size.y;
			corners[3] += brick->size;
			
			
			float sqr_distance;
			for (unsigned int i = 0; i < 4; ++i)
			{
				sqr_distance = ((corners[i].x - position.x) * (corners[i].x - position.x)) +
					((corners[i].y - position.y) * (corners[i].y - position.y));
				
				if (sqr_distance <= sqr_radius)
				{
					ret.position = corners[i];
					
					corners[i] -= position; // now relative
					corners[i] /= (float)sqrt(sqr_distance); // now normal
					
					float dotprod = speed.x * corners[i].x + speed.y * corners[i].y;
					
					speed -= 2.0f * corners[i] * dotprod;
					
					ret.happened = true;
					break;
				}
			}
		}
	}
	
	ret.speed = (old_speed + speed) / 2.0f;
	
	return ret;
}

Ball::Collision Ball::collide(const Paddle* paddle)
{
	float old_speed_len = sqrtf((speed.x * speed.x) + (speed.y * speed.y));
	sf::Vector2f old_speed = speed;
	Collision ret;
	ret.happened = false;
	
	if ((position.x >= (paddle->position.x - paddle->origin.x)) &&
		(position.x <= (paddle->position.x + paddle->size.x - paddle->origin.x)))
	{
		if (position.y <= (paddle->position.y + (paddle->size.y / 2.0f) - paddle->origin.y)) // alatta
		{
			if ((position.y + (size.y / 2.0f)) >= paddle->position.y - paddle->origin.y)
			{
				speed.y = -fabs(speed.y);
				ret.happened = true;
				ret.position = sf::Vector2f(position.x, position.y + size.y / 2.0f);
			}
		}
		else // fölötte
		{
			if ((position.y - (size.y / 2.0f)) <= (paddle->position.y + paddle->size.y  - paddle->origin.y))
			{
				speed.y = fabs(speed.y);
				ret.happened = true;
				ret.position = sf::Vector2f(position.x, position.y - size.y / 2.0f);
			}
		}
	}
	else
	{
		if ((position.y >= (paddle->position.y - paddle->origin.y)) &&
			(position.y <= (paddle->position.y + paddle->size.y - paddle->origin.y)))
		{
			if (position.x <= (paddle->position.x + (paddle->size.x / 2.0f) - paddle->origin.x)) // balra
			{
				if ((position.x + (size.x / 2.0f)) >= paddle->position.x  - paddle->origin.x)
				{
					speed.x = -fabs(speed.x);
					ret.happened = true;
					ret.position = sf::Vector2f(position.x + size.x / 2.0f, position.y);
				}
			}
			else // jobbra
			{
				if ((position.x - (size.x / 2.0f)) <= (paddle->position.x + paddle->size.x  - paddle->origin.x))
				{
					speed.x = fabs(speed.x);
					ret.happened = true;
					ret.position = sf::Vector2f(position.x - size.x / 2.0f, position.y);
				}
			}
		}
		else
		{
			// 4 sarok
			
			float sqr_radius = size.x * size.x * 0.25f; // ez tuti jó
			
			
			sf::Vector2f corners[4];
			
			corners[0] = corners[1] = corners[2] = corners[3] = paddle->position - paddle->origin;
			corners[1].x += paddle->size.x;
			corners[2].y += paddle->size.y;
			corners[3] += paddle->size;
			
			
			float sqr_distance;
			for (unsigned int i = 0; i < 4; ++i)
			{
				sqr_distance = ((corners[i].x - position.x) * (corners[i].x - position.x)) +
					((corners[i].y - position.y) * (corners[i].y - position.y));
				
				if (sqr_distance <= sqr_radius)
				{
					ret.position = corners[i];
					
					corners[i] -= position; // now relative
					corners[i] /= (float)sqrt(sqr_distance); // now normal
					
					float dotprod = speed.x * corners[i].x + speed.y * corners[i].y;
					
					speed -= 2.0f * corners[i] * dotprod;
					ret.happened = true;
					break;
				}
			}
		}
	}
	
	speed.x += (position.x - paddle->position.x) / paddle->size.x * 250.0f;
	
	float new_speed_len = sqrtf((speed.x * speed.x) + (speed.y * speed.y));
	
	speed *= (old_speed_len / new_speed_len);
	
	if ((speed.y <= 5) && (speed.y >= -1))
	{
		speed.y += 10;
	}
	
	new_speed_len = sqrtf((speed.x * speed.x) + (speed.y * speed.y));
	
	speed *= (old_speed_len / new_speed_len);
	
	
	ret.speed = (old_speed + speed) / 2.0f;
	
	return ret;
}

Ball::~Ball()
{
	
}
