/*
 * gameover.cpp - XD-Ball
 * Copyright (c) 2012 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "gameover.hpp"

#include <cmath>
#include <cstdio>
#include <sstream>

#include "resources.hpp"
#include "configmanager.hpp"
#include "scoremanager.hpp"

GameOver::GameOver(unsigned int w, unsigned int h, bool scs, unsigned int scr):
	StateBase(w, h), time(0.0f), entering_name(true), score(scr), success(scs)
{
	gui.size.x = w;
	gui.size.y = h;
	
	GUIElementContainer* square = new GUIElementContainer(4.0f / 3.0f);
	gui.attach(square, 1.0f, GUIElement::SCALE_MODE_SMALLER, 0.5f, 0.5f);
	
	if (success)
	{
		title = new GUIString("BREAKOUT!", "EDITUNDO");
	}
	else
	{
		title = new GUIString("GAME OVER", "EDITUNDO");
	}
	
	square->attach(title, 0.2, GUIString::SCALE_MODE_VERTICAL, 0.5, 1);
	
	sprintf(score_buffer, "YOUR SCORE: %d", score);
	your_score = new GUIString(score_buffer, "EDITUNDO");
	square->attach(your_score, 0.1, GUIString::SCALE_MODE_VERTICAL, 0.5, 0.8);
	
	enter_your_name = new GUIString("ENTER YOUR NAME!", "EDITUNDO");
	square->attach(enter_your_name, 0.1, GUIString::SCALE_MODE_VERTICAL, 0.5, 0.7);
	
	your_name = new GUIString("", "EDITUNDO");
	square->attach(your_name, 0.1, GUIString::SCALE_MODE_VERTICAL, 0.5, 0.6);
	
	
	table = new GUIElementContainer(2.5f);
	square->attach(table, 0.5f, GUIElement::SCALE_MODE_VERTICAL, 0.5f, 0.075f);
	
	
	const std::vector< std::pair< sf::Unicode::Text, sf::Uint32 > >& scores
		= ScoreManager::instance().get_scores();
	
	for (unsigned int i = 0; i < scores.size(); ++i)
	{
		table->attach(new GUIString(scores[i].first, "EDITUNDO"), 0.2,
			GUIElement::SCALE_MODE_VERTICAL, 0.0f, 1.0f - (float)i / 4.0f);
		
		sprintf(score_buffer, "%d", scores[i].second);
		
		table->attach(new GUIString(score_buffer, "EDITUNDO"), 0.2,
			GUIElement::SCALE_MODE_VERTICAL, 1.0f, 1.0f - (float)i / 4.0f);
	}
	
	
	if (ConfigManager::instance().get_bool("music"))
	{
		if (success)
		{
			music.OpenFromMemory((const char*)&(Resources::instance().tracks["NYAN"]->front()), Resources::instance().tracks["NYAN"]->size());
		}
		else
		{
			music.OpenFromMemory((const char*)&(Resources::instance().tracks["SAD"]->front()), Resources::instance().tracks["SAD"]->size());
		}
		
		music.SetLoop(true);
		music.SetVolume(75.0f);
		music.Play();
	}
}

void GameOver::process_events(std::vector< sf::Event >& e)
{
	for (unsigned int i = 0; i < e.size(); ++i)
	{
		switch (e[i].Type)
		{
			case sf::Event::KeyPressed:
			{
				if (e[i].Key.Code == sf::Key::Escape)
				{
					pop();
				}
				if (e[i].Key.Code == sf::Key::Return)
				{
					if (entering_name)
					{
						entering_name = false;
						enter_your_name->visible = false;
						
						your_name->align.y = 0.65;
						
						ScoreManager::instance().submit(name, score);
						
						for (unsigned int i = 0; i < table->children.size(); ++i)
						{
							delete table->children[i];
						}
						table->children.clear();
						
						const std::vector< std::pair< sf::Unicode::Text, sf::Uint32 > >& scores
							= ScoreManager::instance().get_scores();
						
						for (unsigned int i = 0; i < scores.size(); ++i)
						{
							table->attach(new GUIString(scores[i].first, "EDITUNDO"), 0.2,
								GUIElement::SCALE_MODE_VERTICAL, 0.0f, 1.0f - (float)i / 4.0f);
							
							sprintf(score_buffer, "%d", scores[i].second);
							
							table->attach(new GUIString(score_buffer, "EDITUNDO"), 0.2,
								GUIElement::SCALE_MODE_VERTICAL, 1.0f, 1.0f - (float)i / 4.0f);
						}
					}
				}
				if (e[i].Key.Code == 279)
				{
					name = ((sf::Unicode::UTF32String)name).substr(
						0, ((sf::Unicode::UTF32String)name).length() - 1);
					your_name->set_text(name);
				}
			}
			break;
			case sf::Event::TextEntered:
			{
				if (entering_name && (((sf::Unicode::UTF32String)name).length() < 12))
				{
					if ((Resources::instance().fonts["EDITUNDO"]->glyphs.count(e[i].Text.Unicode) > 0)
						&& (e[i].Text.Unicode != 27))
					{
						name = ((sf::Unicode::UTF32String)name).append(
							1, e[i].Text.Unicode);
						your_name->set_text(name);
					}
				}
			}
			break;
			default:
			{
				
			}
			break;
		}
	}
}

void GameOver::render()
{
	background.render();
	gui.render();
}

void GameOver::update(float dt)
{
	time += dt;
	background.update(dt);
	
	float multiplier;
	
	if (success)
	{
		multiplier = 5;
	}
	else
	{
		multiplier = 1;
	}
	
	sf::Color color(
		std::min(std::max(       (float) fabs(fmod(time * multiplier, 6.0f) - 3.0f) - 1.0f,  0.0f), 1.0f) * 255.0f,
		std::min(std::max(1.0f - (float)(fabs(fmod(time * multiplier, 6.0f) - 2.0f) - 1.0f), 0.0f), 1.0f) * 255.0f,
		std::min(std::max(1.0f - (float)(fabs(fmod(time * multiplier, 6.0f) - 4.0f) - 1.0f), 0.0f), 1.0f) * 255.0f);
	
	title->set_color(color);
	
	your_score->set_color(255 - color.r, 255 - color.g, 255 - color.b);
	your_name->set_color(255 - color.r, 255 - color.g, 255 - color.b);
}

void GameOver::resize(const sf::Vector2i& s)
{
	size = s;
}

GameOver::~GameOver()
{
	
}
