/*
 * main.cpp - XD-Ball
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */


#include <iostream>

#include "opengl.hpp"
#include <SFML/Window.hpp>

#include "resources.hpp"
#include "statemanager.hpp"
#include "configmanager.hpp"
#include "scoremanager.hpp"
#include "mainmenu.hpp"

GLuint display_list;

void render_quad()
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glTranslatef(0, 0, -1);
	
	glCallList(display_list);
	
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}


int main(int argc, char* argv[])
{
	sf::Clock main_clock;
	
	
	
	float update_frequency = 60;
	float update_time_accum = 0;
	
	float last_time;
	float current_time;
	
	
	const ConfigManager* cfg_mngr_ptr = new ConfigManager();
	
	
	sf::Window window(sf::VideoMode(800, 600), "XD-Ball",
		 sf::Style::Titlebar | sf::Style::Close, sf::WindowSettings(0, 0, 0));
	
	window.UseVerticalSync(true);
	window.EnableKeyRepeat(false);
	window.ShowMouseCursor(false);
	window.SetFramerateLimit(60);
	
	
	glDisable(GL_DITHER);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_ALPHA_TEST);
	
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 0);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, 800, 0, 600);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	GLuint texture;
	
	glGenTextures(1, &texture);
	
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	
	
	const unsigned int texture_width = 800 / (ConfigManager::instance().get_int("blockyness") + 1);
	const unsigned int texture_height = 600 / (ConfigManager::instance().get_int("blockyness") + 1);
	
	
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, texture_width, texture_height,
		0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
	
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, 0);
	
	
	GLuint fbo;
	glGenFramebuffersEXT(1, &fbo);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo);
	
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, texture, 0);
	
	GLuint fragment_shader = glCreateShader(GL_FRAGMENT_SHADER);
	
	const char* fragment_source =
		"uniform sampler2D frame;"
		"uniform float colors;"
		"uniform float opacity;"
		
		"void main()"
		"	{"
		//"	vec3 color = texture2D(frame, (gl_FragCoord.xy + vec2(sin(gl_FragCoord.y / 25.0) * 2.0, sin(gl_FragCoord.x / 25.0) * 2.0)) / vec2(800.0, 600.0)).rgb;"
		"	vec3 color = texture2D(frame, gl_FragCoord.xy / vec2(800.0, 600.0)).rgb;"
		"	gl_FragColor = vec4(vec3(floor(color.r * colors + 0.5), floor(color.g * colors + 0.5), floor(color.b * colors + 0.5)) / colors, opacity);"
		//"gl_FragColor = vec4(color.r, color.g, color.b, 1.0);"
		"}";
	
	glShaderSource(fragment_shader, 1, &fragment_source, 0);
	glCompileShader(fragment_shader);
	
	char compile_log[1024];
	compile_log[0] = 0;
	glGetInfoLogARB(fragment_shader, sizeof(compile_log), 0, compile_log);
	
	if (compile_log[0])
	{
		std::cout << "Shader compilation log:\n" << compile_log << "\n";
	}
	
	GLuint shader_program = glCreateProgram();
	glAttachShader(shader_program, fragment_shader);
	glLinkProgram(shader_program);
	
	int frame_loc = glGetUniformLocation(shader_program, "frame");
	int colors_loc = glGetUniformLocation(shader_program, "colors");
	int opacity_loc = glGetUniformLocation(shader_program, "opacity");
	
	glUseProgram(shader_program);
	
	glUniform1i(frame_loc, 1);
	glUniform1f(colors_loc, pow(2, ConfigManager::instance().get_int("colorfulness") * 2));
	glUniform1f(opacity_loc, 1.0f - (float)ConfigManager::instance().get_int("motionblur") / 10.0f);
	
	display_list = glGenLists(1);
	glNewList(display_list, GL_COMPILE);
		glBegin(GL_QUADS);
			glVertex2i(-1, -1);
			glVertex2i(1, -1);
			glVertex2i(1, 1);
			glVertex2i(-1, 1);
		glEnd();
	glEndList();
	
	const Resources* res_ptr = new Resources();
	
	const StateManager* st_mngr_ptr = new StateManager();
	StateManager::instance().push_state(new MainMenu(800, 600));
	
	const ScoreManager* scr_mngr_ptr = new ScoreManager();
	
	sf::Event event;
	std::vector< sf::Event > events;
	
	current_time = main_clock.GetElapsedTime();
	last_time = current_time;
	
	
	window.SetCursorPosition(400, 300);
	while (true)
	{
		events.clear();
		while (window.GetEvent(event))
		{
			switch (event.Type)
			{
				case sf::Event::Closed:
				{
					window.Close();
				}
				break;
				case sf::Event::Resized:
				{
					StateManager::instance().resize_all(sf::Vector2i(event.Size.Width, event.Size.Height));
				}
				break;
				default:
				{
					events.push_back(event);
				}
				break;
			}
			
			if (!window.IsOpened())
			{
				break;
			}
		}
		
		window.SetCursorPosition(400, 300);
		
		if (!window.IsOpened())
		{
			break;
		}
		
		StateManager::instance().process_events(events);
		
		last_time = current_time;
		current_time = main_clock.GetElapsedTime();
		
		update_time_accum += (current_time - last_time);
		while (update_time_accum >= (1.0 / update_frequency))
		{
			StateManager::instance().update(1.0 / update_frequency);
			if (StateManager::instance().is_empty())
			{
				break;
			}
			update_time_accum -= (1.0 / update_frequency);
			events.clear();
		}
		
		if (StateManager::instance().is_empty())
		{
			break;
		}
		
		
		// PASS 1
		glUseProgram(0);
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo);
		
		glViewport(0, 0, texture_width, texture_height);
		
		StateManager::instance().render();
		
		// PASS 2
		glUseProgram(shader_program);
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
		
		glViewport(0, 0, 800, 600);
		render_quad();
		
		window.Display();
	}
	
	delete st_mngr_ptr;
	delete cfg_mngr_ptr;
	delete scr_mngr_ptr;
	delete res_ptr;
	
	glDeleteFramebuffersEXT(1, &fbo);
	glDeleteLists(display_list, 1);
	//std::cout << __LINE__ << ".: " << gluErrorString(glGetError()) << "\n";
	return 0;
}
