/*
 * particle.cpp - XD-Ball
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "particle.hpp"

#include <cmath>
#include <iostream>
Particle::Particle(const sf::Vector2f& p, const sf::Vector2f& s, float d, const sf::Color& c):
	Sprite("SPARK", p), time(0), speed(s), duration(d)
{
	size.x = 4;
	origin.x = 2;
	origin.y = 0.0f;
	color = c;
}

void Particle::update(float dt)
{
	time += dt;
	
	position += speed * dt;
	speed.y -= 200.0f * dt;
	speed *= 0.99f;
	
	size.y = sqrtf((speed.x * speed.x) + (speed.y * speed.y)) / 5.0f;
	
	angle = atan2(speed.y, speed.x) * 180.0f / M_PI - 90.0f;
	
	color.a = 255 - (sf::Uint8)(255.0f * time / duration);
}

bool Particle::is_dead()
{
	return time >= duration;
}

Particle::~Particle()
{
	
}
