/*
 * resources.cpp - XD-Ball
 * Copyright (c) 2011, 2012 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "resources.hpp"

#include <fstream>

Resources::Resources()
{
	textures["BALL"] = new Texture("ball.png");
	textures["BRICK"] = new Texture("brick.png");
	textures["PADDLE"] = new Texture("paddle.png");
	textures["SPARK"] = new Texture("spark.png");
	textures["EDITUNDO"] = new Texture("editundo.png");
	
	textures["BACKGROUND1"] = new Texture("background1.png");
	textures["BACKGROUND2"] = new Texture("background2.png");
	
	sounds["KNOCK"] = new sf::SoundBuffer();
	sounds["KNOCK"]->LoadFromFile("knock.wav");
	
	sounds["PADDLE"] = new sf::SoundBuffer();
	sounds["PADDLE"]->LoadFromFile("paddle.wav");
	
	sounds["RICOCHET0"] = new sf::SoundBuffer();
	sounds["RICOCHET0"]->LoadFromFile("ricochet0.wav");
	
	std::ifstream file;
	
	tracks["RICK"] = new std::vector< sf::Uint8 >();
	
	file.open("rick.ogg", std::ios::binary);
	
	file.seekg(0, std::ios::end);
	size_t size = file.tellg();
	file.seekg(0, std::ios::beg);
	
	tracks["RICK"]->resize(size);
	
	file.read((char*)&tracks["RICK"]->front(), size);
	
	file.close();
	
	
	tracks["LOVE"] = new std::vector< sf::Uint8 >();
	
	file.open("love.ogg", std::ios::binary);
	
	file.seekg(0, std::ios::end);
	size = file.tellg();
	file.seekg(0, std::ios::beg);
	
	tracks["LOVE"]->resize(size);
	
	file.read((char*)&tracks["LOVE"]->front(), size);
	
	file.close();
	
	
	tracks["NYAN"] = new std::vector< sf::Uint8 >();
	
	file.open("nyan.ogg", std::ios::binary);
	
	file.seekg(0, std::ios::end);
	size = file.tellg();
	file.seekg(0, std::ios::beg);
	
	tracks["NYAN"]->resize(size);
	
	file.read((char*)&tracks["NYAN"]->front(), size);
	
	file.close();
	
	
	tracks["SAD"] = new std::vector< sf::Uint8 >();
	
	file.open("sad.ogg", std::ios::binary);
	
	file.seekg(0, std::ios::end);
	size = file.tellg();
	file.seekg(0, std::ios::beg);
	
	tracks["SAD"]->resize(size);
	
	file.read((char*)&tracks["SAD"]->front(), size);
	
	file.close();
	
	
	
	std::vector< sf::Uint8 > data;
	file.open("editundo.fnt", std::ios::binary);
	
	file.seekg(0, std::ios::end);
	size = file.tellg();
	file.seekg(0, std::ios::beg);
	
	data.resize(size);
	
	file.read((char*)&data.front(), size);
	
	file.close();
	
	
	fonts["EDITUNDO"] = new Font("EDITUNDO", &data.front(), size);
}

Resources::~Resources()
{
	for (std::map< std::string, Texture* >::iterator i = textures.begin();
		i != textures.end(); ++i)
	{
		delete i->second;
	}
	
	for (std::map< std::string, sf::SoundBuffer* >::iterator i = sounds.begin();
		i != sounds.end(); ++i)
	{
		delete i->second;
	}
	
	for (std::map< std::string, std::vector< sf::Uint8 >* >::iterator i = tracks.begin();
		i != tracks.end(); ++i)
	{
		delete i->second;
	}
	
	for (std::map< std::string, Font* >::iterator i = fonts.begin();
		i != fonts.end(); ++i)
	{
		delete i->second;
	}
}
