/*
 * scoremanager.cpp - Kukatz 3D
 * Copyright (c) 2012 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *   2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *   3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "scoremanager.hpp"

#include <fstream>

#include <SFML/Config.hpp>

ScoreManager::ScoreManager()
{
	std::ifstream file("highscores", std::ios::binary);
	
	if (file.is_open())
	{
		file.seekg(0, std::ios::end);
		size_t file_length = file.tellg();
		file.seekg(0, std::ios::beg);
		
		if (file_length > 0)
		{
			sf::Uint8 num_scores;
			file.read((char*)(&num_scores), 1);
			
			scores.resize(num_scores);
			
			sf::Uint32 buffer[256];
			sf::Uint8 name_length;
			sf::Uint32 points;
			for (unsigned int i = 0; i < num_scores; ++i)
			{
				file.read((char*)&name_length, 1);
				file.read((char*)buffer, name_length * 4);
				buffer[name_length] = 0;
				
				scores[i].first = buffer;
				
				file.read((char*)&points, 4);
				scores[i].second = points;
			}
		}
	}
}

bool ScoreManager::submit(const sf::Unicode::Text& n, sf::Uint32 s)
{
	bool ret = false;
	
	int before_this = scores.size();
	
	for (unsigned int i = 0; i < scores.size(); ++i)
	{
		if (scores[i].second < s)
		{
			before_this = i;
			ret = true;
			break;
		}
	}
	
	scores.push_back(std::make_pair("", 0));
	
	for (int i = scores.size() - 2; i >= before_this; --i)
	{
		scores[i + 1].first = scores[i].first;
		scores[i + 1].second = scores[i].second;
	}
	
	scores[before_this].first = n;
	scores[before_this].second = s;
	
	if (scores.size() > 5)
	{
		scores.resize(5);
	}
	
	save();
	return ret;
}

const std::vector< std::pair< sf::Unicode::Text, sf::Uint32 > >& ScoreManager::get_scores() const // RESPECT Z FOR THE CONST!
{
	return scores;
}

void ScoreManager::save()
{
	std::ofstream file("highscores", std::ios::binary | std::ios::trunc);
	
	sf::Uint8 scores_size = scores.size();
	
	if (file.is_open())
	{
		file.write((char*)(&scores_size), 1);
		
		sf::Uint8 name_length;
		for (unsigned int i = 0; i < scores_size; ++i)
		{
			sf::Unicode::UTF32String name = scores[i].first;
			name_length = name.length();
			file.write((char*)&name_length, 1);
			file.write((char*)&(name[0]), name_length * 4);
			
			file.write((char*)&scores[i].second, 4);
		}
	}
}

ScoreManager::~ScoreManager()
{
	save();
}
