/*
 * selector.cpp
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *   2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *   3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "selector.hpp"

Selector::Selector(const sf::Unicode::Text& l,
		const std::vector< sf::Unicode::Text >& o,
		unsigned int co, bool ta, float ar, float s):
	turnaround(ta), current_option(co), separator(s)
{
	aspect_ratio = ar;
	
	if (current_option >= o.size())
	{
		current_option = o.size() - 1;
	}
	
	label_container = new GUIElementContainer((aspect_ratio - 0.5f) * separator);
	attach(label_container, 1, SCALE_MODE_VERTICAL, 0, 0.5);
	
	label = new GUIString(l, "EDITUNDO");
	label_container->attach(label, 1, SCALE_MODE_SMALLER, 0, 0.5);
	
	
	leftarrow_container = new GUIElementContainer(0.5f);
	attach(leftarrow_container, 1.0f, SCALE_MODE_VERTICAL, separator, 0.5f);
	
	leftarrow_single = new GUIString(L"<", "EDITUNDO");
	leftarrow_container->attach(leftarrow_single, 1.0f, SCALE_MODE_SMALLER,
		0.5f, 0.5f);
	
	leftarrow_double = new GUIString(L"<", "EDITUNDO");
	leftarrow_container->attach(leftarrow_double, 1.0f, SCALE_MODE_SMALLER,
		0.5f, 0.5f);
	
	
	rightarrow_container = new GUIElementContainer(0.5f);
	attach(rightarrow_container, 1.0f, SCALE_MODE_VERTICAL, 1.0f, 0.5f);
	
	rightarrow_single = new GUIString(L">", "EDITUNDO");
	rightarrow_container->attach(rightarrow_single, 1.0f, SCALE_MODE_SMALLER,
		0.5f, 0.5f);
	
	rightarrow_double = new GUIString(L">", "EDITUNDO");
	rightarrow_container->attach(rightarrow_double, 1.0f, SCALE_MODE_SMALLER,
		0.5f, 0.5f);
	
	option_container = new GUIElementContainer(
		(aspect_ratio - 0.5f) * (1.0f - separator) - 0.5f);
	attach(option_container, 1.0f, SCALE_MODE_VERTICAL,
		(aspect_ratio / 2.0f + 0.25f) /
			((aspect_ratio + 0.75f) - aspect_ratio / 2.0f),
		0.5);
	
	GUIString* str;
	for(size_t i = 0; i < o.size(); ++i)
	{
		str = new GUIString(std::string(o[i]), "EDITUNDO");
		str->visible = (i == current_option);
		
		options.push_back(str);
		option_container->attach(str, 1, SCALE_MODE_SMALLER, 0.5, 0.5);
	}
	
	update_arrows();
}

void Selector::update_arrows()
{
	leftarrow_single->visible = leftarrow_double->visible =
		rightarrow_single->visible = rightarrow_double->visible = false;
	
	if (options.size() > 1)
	{
		if (turnaround)
		{
			if (options.size() > 2)
			{
				leftarrow_double->visible = true;
				rightarrow_double->visible = true;
			}
			else
			{
				leftarrow_single->visible = true;
				rightarrow_single->visible = true;
			}
		}
		else
		{
			if (current_option > 0)
			{
				if (current_option == 1)
				{
					leftarrow_single->visible = true;
				}
				else
				{
					leftarrow_double->visible = true;
				}
			}
			
			if (current_option < options.size() - 1)
			{
				if(current_option == options.size() - 2)
				{
					rightarrow_single->visible = true;
				}
				else
				{
					rightarrow_double->visible = true;
				}
			}
		}
	}
}

void Selector::render()
{
	leftarrow_container->align.x = separator;
	
	label_container->aspect_ratio = (aspect_ratio - 0.5f) * separator;
	
	option_container->aspect_ratio =
		(aspect_ratio - 0.5f) * (1.0f - separator) - 0.5f;
	
	option_container->align.x =
		(option_container->aspect_ratio - aspect_ratio + 0.5f) / 
		(option_container->aspect_ratio - aspect_ratio);
	
	GUIElementContainer::render();
}

void Selector::set_color(sf::Uint8 r, sf::Uint8 g, sf::Uint8 b, sf::Uint8 a)
{
	label->set_color(r, g, b, a);
	leftarrow_single->set_color(r, g, b, a);
	leftarrow_double->set_color(r, g, b, a);
	rightarrow_single->set_color(r, g, b, a);
	rightarrow_double->set_color(r, g, b, a);
	
	for(size_t i = 0; i < options.size(); ++i)
	{
		options[i]->set_color(r, g, b, a);
	}
}

unsigned int Selector::get_option()
{
	return current_option;
}

void Selector::next()
{
	options[current_option]->visible = false;
	if (current_option < options.size() - 1)
	{
		++current_option;
	}
	else
	{
		if (turnaround)
		{
			current_option = 0;
		}
	}
	options[current_option]->visible = true;
	
	update_arrows();
}

void Selector::prev()
{
	options[current_option]->visible = false;
	if (current_option > 0)
	{
		--current_option;
	}
	else
	{
		if (turnaround)
		{
			current_option = options.size() -1;
		}
	}
	options[current_option]->visible = true;
	
	update_arrows();
}

Selector::~Selector()
{
	// no need to delete the items, the container destructor will delete them
}
