/*
 * singleton.inl
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *   2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *   3. This notice may not be removed or altered from any source
 *      distribution.
 */

// This file is based on OGRE's OgreSingleton.h which can be found here:
// http://ogre.svn.sourceforge.net/viewvc/ogre/branches/v1-7/OgreMain/include/OgreSingleton.h?revision=9527&view=markup
// Thanks!

#include <cassert>

template< class T >
T* Singleton< T >::msp_instance = 0;

template< class T>
Singleton< T >::Singleton()
{
	assert(!msp_instance);
	msp_instance = static_cast< T* >(this);
}

template< class T >
T& Singleton< T >::instance()
{
	assert(msp_instance);
	return *msp_instance;
}

template< class T >
T* Singleton< T >::instance_ptr()
{
	return msp_instance;
}

template< class T >
T& Singleton< T >::operator () ()
{
	return &msp_instance;
}

template< class T >
Singleton< T >::~Singleton() // DO NOT have to delete msp_instance, it is
{                            // being deconstructed right now...
	assert(msp_instance);
	msp_instance = 0;
}
