/*
 * statemanager.cpp - Kukatz 3D
 * Copyright (c) 2010, 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *	 1. The origin of this software must not be misrepresented; you must not
 *			claim that you wrote the original software. If you use this software
 *			in a product, an acknowledgment in the product documentation would be
 *			appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *			misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *			distribution.
 */

#include "statemanager.hpp"

#include "statebase.hpp"

StateManager::StateManager()
{
	
}

void StateManager::resize_all(const sf::Vector2i& size)
{
	for (size_t i = 0; i < state_stack.size(); ++i)
	{
		state_stack[i]->resize(size);
	}
}

StateBase* StateManager::get_top_state()
{
	if (state_stack.empty())
	{
		return 0;
	}
	else
	{
		return state_stack.back();
	}
}

void StateManager::push_state(StateBase* state)
{
	if (state != 0)
	{
		state_stack.push_back(state);
	}
}

void StateManager::clear()
{
	while (!state_stack.empty())
	{
		delete state_stack.back();
		state_stack.pop_back();
	}
}

bool StateManager::is_empty()
{
	return state_stack.empty();
}

void StateManager::process_events(std::vector< sf::Event >& events)
{
	if (!state_stack.empty())
	{
		state_stack.back()->process_events(events);
	}
}

void StateManager::update(float dt)
{
	for(std::vector< StateBase* >::iterator i = state_stack.begin(); i < state_stack.end(); ++i)
	{
		if ((*i)->should_pop())
		{
			delete (*i);
			state_stack.erase(i);
		}
	}
	
	if (!state_stack.empty())
	{
		state_stack.back()->update(dt);
	}
}

void StateManager::render()
{
	if (!state_stack.empty())
	{
		state_stack.back()->render();
	}
}

StateManager::~StateManager()
{
	clear();
}
